#!/bin/bash

if [[ -z $HOME ]]; then
	HOME=/root
fi

PATH=/sbin:"$PATH"

cidr2mask() {
	local i mask=" "
	local full_octets=$(($1/8))
	local partial_octet=$(($1%8))

	for ((i=0; i<4; i++)); do
		if [ $i -lt $full_octets ]; then
			mask+=255
		elif [ $i -eq $full_octets ]; then
			mask+=$((256 - 2**(8-$partial_octet)))
		else
			mask+=0
		fi
		test $i -lt 3 && mask+=.
	done

	echo $mask
}

mac2iface() {
	find /sys/class/net -mindepth 1 -maxdepth 1 ! -name lo -execdir grep -wq $1 {}/address \; -printf "%P"
}

replace_deb_url() {
	file="$1"
	new_url="$2"

	if [[ -n $file && -n $new_url ]]; then
		# Replace URL in each line with $new_url, unless the line contains 'security'
		sed -r -i "\|security|b; s| http[^ ]+ | $new_url |g" "$file"
	fi
}
ctx_authorizedkeys() {
	if [[ ! -z ${SSHKEYS+x} ]]; then
		echo "SSHKEYS"

		mkdir -p "$HOME/.ssh"
		keys_file="$HOME/.ssh/authorized_keys"
		touch "$keys_file"

		NEW_CONTENTS=$(sed '/context-generated/d' "$keys_file") # Delete all lines that contain 'context-generated'
		while read -r key; do # Iterate on each line in SSHKEYS
			if [[ ! -z ${key} ]]; then
				NEW_CONTENTS="$NEW_CONTENTS\n$key context-generated"
			fi
		done <<< "$SSHKEYS"
		echo -e "$NEW_CONTENTS" > "$keys_file"
	fi
}


ctx_firstboot() {
	if [ -f /.firstboot ]; then
		partition=$(mount -v | fgrep 'on / ' | cut -d\  -f1)

		echo "RESIZING FILESYSTEM $partition"
		/sbin/resize2fs "$partition"

		ctx_hostkeys

		if [ -f /etc/ssh/sshd_not_to_be_run ]; then
			rm /etc/ssh/sshd_not_to_be_run
			systemctl restart ssh
		fi

		rm /.firstboot
	fi
}


ctx_hostkeys() {
	#setup new hostkeys
	if [ ! -f /etc/ssh/ssh_host_rsa_key ]
	then
        echo "Generating SSH hostkeys"
        ssh-keygen -A
	fi
}

ctx_hostname() {
	if [[ ! -z ${HOSTNAME} ]]; then
		echo "HOSTNAME"
		hostname ${HOSTNAME}
		echo "$HOSTNAME" > /etc/hostname

		sed -i /\#Context-Generated/d /etc/hosts
		echo "127.0.0.1 $HOSTNAME #Context-Generated" >> /etc/hosts
	fi
}


ctx_nameservers() {
    # nameservers implemented in ctx_networking
    :
}

networksetup() {
	local config="$1"
	local iface="$2"
	local gateway_ip="$3"

	echo "auto $iface"

	if [[ "$config" == "dhcp" ]]; then
		echo "iface $iface inet dhcp"
		return
	elif [[ -z $config ]]; then
		echo "iface $iface inet manual"
		return
	fi

	local split=(${config//// })
	local ip=${split[0]}
	local cidr=${split[1]}
	local mask=$(cidr2mask $cidr)
	echo "iface $iface inet static"
	echo "  address $ip"
	echo "  netmask $mask"

	if [[ -n $gateway_ip ]]; then
		echo "  gateway $gateway_ip metric $metric"
		metric=`expr $metric + 1`
	fi
}

ctx_networking() {
	if [[ ! -z ${IFACE_NUM} ]]; then
		if [[ -e "/run/network/ifstate" && -s "/run/network/ifstate" ]]; then
			output="NETWORKING"
			INTERFACES=$(
				echo "### CONTEXT START ###"
				echo -e "# Warning, this content is automatically generated and will be overwritten\n"
				echo "auto lo"
				echo "iface lo inet loopback"
				echo ""
				metric=1
				for (( i=0; i < ${IFACE_NUM}; i++))
				do
					key1="IFACE${i}_CONFIG"
					config=${!key1}
					key2="IFACE${i}_GATEWAY"
					gateway=${!key2}
					key3="IFACE${i}_MAC"
					mac=${!key3}
					iface=$(mac2iface $mac)
					networksetup "$config" "$iface" "$gateway"

					x=1
					while :; do
						key="IFACE${i}_${x}_CONFIG"
						sub_config=${!key}
						if [[ -n $sub_config ]]; then
							echo ""
							networksetup "$sub_config" "$iface:$x"
						else
							break
						fi
						((x++))
					done
					echo ""
				done
				echo "source /etc/network/interfaces.d/*.conf"
				echo "### CONTEXT END ###"
			)
			echo -e "$output"

			OLD_COMPARE=$(cat /etc/network/interfaces | tr '\r' ' ')
			NEW_COMPARE=$(echo "$INTERFACES" | tr '\r' ' ')

			if [[ "$OLD_COMPARE" != "$NEW_COMPARE" ]]; then
				echo "Network changing"
				ifdown -a --force
				echo "$INTERFACES" > /etc/network/interfaces
				ifup -a --force
			else
				echo "Network had no changes"
			fi
		else
			output="NETWORKING"
			INTERFACES=$(
				echo "### CONTEXT START ###"
				echo -e "# Warning, this content is automatically generated and will be overwritten\n"
				echo "network:"
				echo "  version: 2"
				echo "  renderer: networkd"
				echo "  ethernets:"
				metric=1
				for (( i=0; i < ${IFACE_NUM}; i++))
				do
					key1="IFACE${i}_CONFIG"
					config=${!key1}
					key2="IFACE${i}_GATEWAY"
					gateway=${!key2}
					key3="IFACE${i}_MAC"
					mac=${!key3}
					iface=$(mac2iface $mac)

					echo "    $iface:"

					if [[ "$config" == "dhcp" ]]; then
						echo "      dhcp4: yes"
					else
						echo "      dhcp4: no"
					fi

					echo "      addresses:"
					echo "        - $config"

					x=1
					while :; do
						key="IFACE${i}_${x}_CONFIG"
						sub_config=${!key}
						if [[ -n $sub_config ]]; then
							echo "        - $sub_config"
						else
							break
						fi
						((x++))
					done
					if [[ ! -z ${gateway} ]]; then
						echo "      routes:"
						echo "       - to: default"
						echo "         via: $gateway"
					fi

					if [[ ! -z ${NAMESERVERS} ]]; then
						echo "      nameservers:"
						echo "        addresses:"
						for host in ${NAMESERVERS}; do
							echo "          - $host"
						done
					fi

					if [[ ! -z ${mac} ]]; then
						echo "      match:"
						echo "        macaddress: $mac"
					fi
				done
				echo -e "\n### CONTEXT END ###"
			)
			echo -e "$output"

			OLD_COMPARE=$(cat /etc/netplan/01-netcfg.yaml | tr '\r' ' ')
			NEW_COMPARE=$(echo "$INTERFACES" | tr '\r' ' ')

			if [[ "$OLD_COMPARE" != "$NEW_COMPARE" ]]; then
				echo "Network changing"
				echo "$INTERFACES" > /etc/netplan/01-netcfg.yaml
				chmod 600 /etc/netplan/01-netcfg.yaml
				netplan apply
			else
				echo "Network had no changes"
			fi
		fi
	fi
}

ctx_repourls() {
  if [[ ! -z ${REPO_BASE_URL} ]]; then
    echo "REPOURLS"
    replace_deb_url /etc/apt/sources.list "${REPO_BASE_URL}/ubuntu"
  fi
  if [[ ! -z ${CONTEXT_REPO_BASE_URL} ]] && [[ -f /etc/apt/sources.list.d/context.list ]]; then
    replace_deb_url /etc/apt/sources.list.d/context.list "${CONTEXT_REPO_BASE_URL}/ubuntu"
  fi
}

